; G8RTOS_SchedulerASM.s
; Holds all ASM functions needed for the scheduler
; Note: If you have an h file, do not have a C file and an S file of the same name

	; Functions Defined
	.def G8RTOS_Start, PendSV_Handler

	; Dependencies
	.ref CurrentlyRunningThread, G8RTOS_Scheduler

	.thumb		; Set to thumb mode
	.align 2	; Align by 2 bytes (thumb mode uses allignment by 2 or 4)
	.text		; Text section

; Need to have the address defined in file 
; (label needs to be close enough to asm code to be reached with PC relative addressing)
RunningPtr: .field CurrentlyRunningThread, 32

; G8RTOS_Start
;	Sets the first thread to be the currently running thread
;	Starts the currently running thread by setting Link Register to tcb's Program Counter
G8RTOS_Start:

	.asmfunc
	LDR R0, RunningPtr	;LOAD in *StackPointer from the first thread into SP
	LDR R1, [R0]
	LDR SP, [R1]

	;Load currently running thread's contect into CPU
	POP {R4-R11}
	POP {R0-R3}
	POP {R12}
	ADD SP, SP, #4	;DISCARD LR FROM INITIAL
	POP {LR}		;pop PC into LR
	ADD SP, SP, #4	;DISCARD PSR (why tho?)

	;Enable interrupt
	CPSIE I 	;If an interrupt was already in the pending state, the processor accepts the interrupt after the CPSIE I is executed
	BX LR
	.endasmfunc

; PendSV_Handler
; - Performs a context switch in G8RTOS
; 	- Saves remaining registers into thread stack
;	- Saves current stack pointer to tcb
;	- Calls G8RTOS_Scheduler to get new tcb
;	- Set stack pointer to new stack pointer from new tcb
;	- Pops registers from thread stack
PendSV_Handler:
	
	.asmfunc
	CPSID I 			;Disable interrupts during context switch
	PUSH {R4-R11}		;Save R4-R11 onto the stack of the current thread

	LDR R0, RunningPtr
	LDR R1, [R0]
	STR SP, [R1]		;SAVE current stack pointer in *StackPointer of current TCB

	PUSH {LR}			;SAVE LR
	BL G8RTOS_Scheduler
	POP {LR}			;RETAIN 0xFFFFFFF9 in LR so program will exit handler mode

	LDR R0, RunningPtr	;LOAD *StackPointer from tcb_t of the new thread
	LDR R1, [R0]
	LDR SP, [R1]		;LOAD the new thread's Stack pointer

	POP {R4-R11}
	CPSIE I 			;Enable interrupts after context switch
	BX LR
	.endasmfunc
	
	; end of the asm file
	.align
	.end
