/*
 * G8RTOS_Structure.h
 *
 *  Created on: Jan 12, 2017
 *      Author: Raz Aloni
 */

#ifndef G8RTOS_STRUCTURES_H_
#define G8RTOS_STRUCTURES_H_
#include "G8RTOS.h"

#define MAX_NAME_LENGTH 16
/*********************************************** Data Structure Definitions ***********************************************************/

/*
 *  Thread Control Block:
 *      - Every thread has a Thread Control Block
 *      - The Thread Control Block holds information about the Thread Such as the Stack Pointer, Priority Level, and Blocked Status
 *      - For Lab 2 the TCB will only hold the Stack Pointer, next TCB and the previous TCB (for Round Robin Scheduling)
 */

typedef struct TCB{
    int32_t *StackPointer;
    struct TCB *NextTCB;
    struct TCB *PreviousTCB;
    char threadName[MAX_NAME_LENGTH];
    bool isAlive;
    uint8_t Priority;
    bool Asleep;
    uint32_t SleepCount;
    semaphore_t *Blocked;
    threadId_t ThreadID;
}tcb_t;


/*
 *  Periodic Thread Control Block:
 *      - Holds a function pointer that points to the periodic thread to be executed
 *      - Has a period in us
 *      - Holds Current time
 *      - Contains pointer to the next periodic event - linked list
 */
typedef struct PTCB{
    void (*Handler)(void);
    uint32_t Period;
    uint32_t ExecuteTime;
    uint32_t CurrentTime;
    struct PTCB *NextPTCB;
    struct PTCB *PreviousPTCB;
}ptcb_t;
/*********************************************** Data Structure Definitions ***********************************************************/


/*********************************************** Public Variables *********************************************************************/

tcb_t * CurrentlyRunningThread;
tcb_t * TempNextThread;
tcb_t * HighestPriorityHead;
tcb_t * LowestPriorityTail;
/*********************************************** Public Variables *********************************************************************/




#endif /* G8RTOS_STRUCTURES_H_ */
